"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessageReceiver = void 0;
const tsyringe_1 = require("tsyringe");
const error_1 = require("../error");
const ConnectionService_1 = require("../modules/connections/services/ConnectionService");
const JsonTransformer_1 = require("../utils/JsonTransformer");
const messageType_1 = require("../utils/messageType");
const AgentConfig_1 = require("./AgentConfig");
const Dispatcher_1 = require("./Dispatcher");
const EnvelopeService_1 = require("./EnvelopeService");
const TransportService_1 = require("./TransportService");
const InboundMessageContext_1 = require("./models/InboundMessageContext");
let MessageReceiver = class MessageReceiver {
    constructor(config, envelopeService, transportService, connectionService, dispatcher) {
        this.inboundTransports = [];
        this.config = config;
        this.envelopeService = envelopeService;
        this.transportService = transportService;
        this.connectionService = connectionService;
        this.dispatcher = dispatcher;
        this.logger = this.config.logger;
    }
    registerInboundTransport(inboundTransport) {
        this.inboundTransports.push(inboundTransport);
    }
    /**
     * Receive and handle an inbound DIDComm message. It will unpack the message, transform it
     * to it's corresponding message class and finally dispatch it to the dispatcher.
     *
     * @param inboundPackedMessage the message to receive and handle
     */
    async receiveMessage(inboundPackedMessage, session) {
        if (typeof inboundPackedMessage !== 'object' || inboundPackedMessage == null) {
            throw new error_1.AriesFrameworkError('Invalid message received. Message should be object');
        }
        this.logger.debug(`Agent ${this.config.label} received message`);
        const unpackedMessage = await this.unpackMessage(inboundPackedMessage);
        const senderKey = unpackedMessage.senderVerkey;
        const recipientKey = unpackedMessage.recipientVerkey;
        let connection = undefined;
        if (senderKey && recipientKey) {
            // TODO: only attach if theirKey is present. Otherwise a connection that may not be complete, validated or correct will
            // be attached to the message context. See #76
            connection = (await this.connectionService.findByVerkey(recipientKey)) || undefined;
            // We check whether the sender key is the same as the key we have stored in the connection
            // otherwise everyone could send messages to our key and we would just accept
            // it as if it was send by the key of the connection.
            if (connection && connection.theirKey != null && connection.theirKey != senderKey) {
                throw new error_1.AriesFrameworkError(`Inbound message 'sender_key' ${senderKey} is different from connection.theirKey ${connection.theirKey}`);
            }
        }
        this.logger.info(`Received message with type '${unpackedMessage.message['@type']}' from connection ${connection === null || connection === void 0 ? void 0 : connection.id} (${connection === null || connection === void 0 ? void 0 : connection.theirLabel})`, unpackedMessage.message);
        const message = await this.transformMessage(unpackedMessage);
        const messageContext = new InboundMessageContext_1.InboundMessageContext(message, {
            connection,
            senderVerkey: senderKey,
            recipientVerkey: recipientKey,
        });
        // We want to save a session if there is a chance of returning outbound message via inbound transport.
        // That can happen when inbound message has `return_route` set to `all` or `thread`.
        // If `return_route` defines just `thread`, we decide later whether to use session according to outbound message `threadId`.
        if (senderKey && recipientKey && message.hasAnyReturnRoute() && session) {
            const keys = {
                recipientKeys: [senderKey],
                routingKeys: [],
                senderKey: recipientKey,
            };
            session.keys = keys;
            session.inboundMessage = message;
            session.connection = connection;
            this.transportService.saveSession(session);
        }
        await this.dispatcher.dispatch(messageContext);
    }
    /**
     * Unpack a message using the envelope service.
     * If message is not packed, it will be returned as is, but in the unpacked message structure
     *
     * @param packedMessage the received, probably packed, message to unpack
     */
    async unpackMessage(packedMessage) {
        // If the inbound message has no @type field we assume
        // the message is packed and must be unpacked first
        if (!this.isUnpackedMessage(packedMessage)) {
            try {
                return await this.envelopeService.unpackMessage(packedMessage);
            }
            catch (error) {
                this.logger.error('error while unpacking message', {
                    error,
                    packedMessage,
                    errorMessage: error.message,
                });
                throw error;
            }
        }
        // If the message does have an @type field we assume
        // the message is already unpacked an use it directly
        else {
            const unpackedMessage = { message: packedMessage };
            return unpackedMessage;
        }
    }
    isUnpackedMessage(message) {
        return '@type' in message;
    }
    /**
     * Transform an unpacked DIDComm message into it's corresponding message class. Will look at all message types in the registered handlers.
     *
     * @param unpackedMessage the unpacked message for which to transform the message in to a class instance
     */
    async transformMessage(unpackedMessage) {
        // replace did:sov:BzCbsNYhMrjHiqZDTUASHg;spec prefix for message type with https://didcomm.org
        messageType_1.replaceLegacyDidSovPrefixOnMessage(unpackedMessage.message);
        const messageType = unpackedMessage.message['@type'];
        const MessageClass = this.dispatcher.getMessageClassForType(messageType);
        if (!MessageClass) {
            throw new error_1.AriesFrameworkError(`No handler for message type "${messageType}" found`);
        }
        // Cast the plain JSON object to specific instance of Message extended from AgentMessage
        const message = JsonTransformer_1.JsonTransformer.fromJSON(unpackedMessage.message, MessageClass);
        return message;
    }
};
MessageReceiver = __decorate([
    tsyringe_1.scoped(tsyringe_1.Lifecycle.ContainerScoped),
    __metadata("design:paramtypes", [AgentConfig_1.AgentConfig,
        EnvelopeService_1.EnvelopeService,
        TransportService_1.TransportService,
        ConnectionService_1.ConnectionService,
        Dispatcher_1.Dispatcher])
], MessageReceiver);
exports.MessageReceiver = MessageReceiver;
//# sourceMappingURL=MessageReceiver.js.map